package com.pdftron.android.tutorial.customui;

import android.app.ProgressDialog;
import android.content.Context;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;

import com.pdftron.common.PDFNetException;
import com.pdftron.pdf.PDFDoc;
import com.pdftron.pdf.PDFViewCtrl;
import com.pdftron.pdf.config.ViewerBuilder2;
import com.pdftron.pdf.config.ViewerConfig;
import com.pdftron.pdf.controls.PdfViewCtrlTabHostFragment2;
import com.pdftron.pdf.controls.ThumbnailsViewFragment;
import com.pdftron.pdf.dialog.pdflayer.PdfLayerDialog;
import com.pdftron.pdf.dialog.pdflayer.PdfLayerUtils;
import com.pdftron.pdf.model.FileInfo;
import com.pdftron.pdf.utils.AppUtils;

import java.util.ArrayList;

import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;


public class SamplePDFViewer extends AppCompatActivity implements PdfViewCtrlTabHostFragment2.TabHostListener {

    private PdfViewCtrlTabHostFragment2 mPdfViewCtrlTabHostFragment;
    private byte[] pdfbytes;
    private PDFViewCtrl pdfViewCtrl;
    AppCompatActivity context;
    private PDFDoc mPdfDoc;
    private PDFDoc mDocToLoad;
    private Uri fileLink;
    private String layerSampleUrl = "http://www.pdfill.com/example/pdf_layer_new.pdf";
    private String attachmentsUrl = "https://tcpdf.org/files/examples/example_041.pdf";
    private String blankPdfUrl = "file:///android_asset/blank.pdf";
    private String TAG = "SamplePDFViewer";
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        try {
            AppUtils.initializePDFNetApplication(this.getApplicationContext());
        } catch (PDFNetException e) {
            e.printStackTrace();
        }
        context = this;
        setContentView(R.layout.pdfcomponent);

        try {
            fileLink = Uri.parse(blankPdfUrl);
            createCustomToolbarFragment(fileLink);
            getSupportFragmentManager().beginTransaction()
                    .replace(R.id.fragment_container, mPdfViewCtrlTabHostFragment)
                    .commit();

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

 // Method used to initialize the viewer fragment with a custom toolbar.
    public void createCustomToolbarFragment(@NonNull Uri fileUri) {
        ThumbnailsViewFragment.FilterModes[] filterModes =new ThumbnailsViewFragment.FilterModes[2];
        filterModes[0] = ThumbnailsViewFragment.FilterModes.BOOKMARKED;
        filterModes[1] = ThumbnailsViewFragment.FilterModes.ANNOTATED;
        ViewerConfig viewerConfig = new ViewerConfig.Builder()
                .toolbarTitle("PDFSample")
                .maximumTabCount(1)
                .multiTabEnabled(false)
                .documentEditingEnabled(false)
                .longPressQuickMenuEnabled(false)
                .thumbnailViewEditingEnabled(false)
                .showEditPagesOption(false)
                .showEditMenuOption(false)
                .showThumbnailView(false)
                .showAnnotationToolbarOption(false)
                .showAnnotationsList(false)
                .showToolbarSwitcher(false)
                .showReflowOption(false)
                .showBottomToolbar(true)
                .showUserBookmarksList(false)
                .userBookmarkCreationEnabled(false)
                .showBookmarksView(true)
                .showPageNumberIndicator(true)
                .showOutlineList(true)
                .annotationsListEditingEnabled(false)
                .permanentPageNumberIndicator(true)
                .hideThumbnailFilterModes(filterModes)
                .build();

        mPdfViewCtrlTabHostFragment = ViewerBuilder2.withUri(fileUri)
                // Specify a custom toolbar
                .usingCustomToolbar(new int[]{R.menu.my_custom_options_toolbar})
                .usingCacheFolder(false)
                // Specify a custom navigation component
                .usingTheme(R.style.PDFTheme)
                .usingConfig(viewerConfig)
                .usingTabClass(MyTabFragment.class)
                .build(this);
        mPdfViewCtrlTabHostFragment.addHostListener(this);
        new DownloadFileTask(this,attachmentsUrl).execute();
    }

    private void downloadPDF(){
    }

    @Override
    public void onTabHostShown() {

    }

    @Override
    public void onTabHostHidden() {

    }

    @Override
    public void onLastTabClosed() {

    }

    @Override
    public void onTabChanged(String tag) {

    }

    @Override
    public boolean onOpenDocError() {
        return false;
    }

    @Override
    public void onNavButtonPressed() {
        mPdfViewCtrlTabHostFragment.removeTabAt(0);//getCurrentPdfViewCtrlFragment().removeFromRecentList();
        finish();
    }
    @Override
    public void onShowFileInFolder(String fileName, String filepath, int itemSource) {

    }

    @Override

    public boolean canShowFileInFolder() {
        return false;
    }

    @Override
    public boolean canShowFileCloseSnackbar() {
        return false;
    }

    @Override
    public boolean onToolbarCreateOptionsMenu(Menu menu, MenuInflater inflater) {

        inflater.inflate(R.menu.my_custom_options_toolbar, menu);

        MenuItem menu_file_attachments = menu.findItem(R.id.menu_attachments);
        MenuItem menu_layers = menu.findItem(R.id.menu_layers);

        try {
            ArrayList<PdfLayerUtils.LayerInfo> layers = PdfLayerUtils.getLayers(pdfViewCtrl, mPdfDoc);
            if (layers!=null && layers.size()>0)
                menu_layers.setVisible(true);
            else
                menu_layers.setVisible(false);

        }catch (Exception e){
            menu_layers.setVisible(false);
            e.printStackTrace();
        }
        if(mDocToLoad!=null)
            mPdfDoc = mDocToLoad;
        if(mPdfDoc!=null && com.pdftron.pdf.utils.Utils.hasFileAttachments(mPdfDoc))
            menu_file_attachments.setVisible(true);
        else
            menu_file_attachments.setVisible(false);
        return false;
    }

    @Override
    public boolean onToolbarPrepareOptionsMenu(Menu menu) {
        return false;
    }

    @Override
    public boolean onToolbarOptionsItemSelected(MenuItem item) {
        pdfViewCtrl = mPdfViewCtrlTabHostFragment.getCurrentPdfViewCtrlFragment().getPDFViewCtrl();
        if(item.getItemId() == android.R.id.home){
            onBackPressed();
            return true;
        }
        if (item.getItemId() == R.id.menu_layers) {
            if(pdfViewCtrl!=null)
                showLayers(this,pdfViewCtrl);
        }
        if (item.getItemId() == R.id.menu_attachments) {
            mPdfViewCtrlTabHostFragment.getCurrentPdfViewCtrlFragment().handleViewFileAttachments();
        }
        return false;
    }

    public void showLayers(Context context, PDFViewCtrl pdfViewCtrl) {
        PdfLayerDialog pdfLayerDialog = new PdfLayerDialog(
                context, pdfViewCtrl);
        pdfLayerDialog.show();
    }
    @Override
    public void onStartSearchMode() {

    }

    @Override
    public void onExitSearchMode() {

    }

    @Override
    public boolean canRecreateActivity() {
        return false;
    }

    @Override
    public void onTabPaused(FileInfo fileInfo, boolean isDocModifiedAfterOpening) {

    }

    @Override
    public void onJumpToSdCardFolder() {

    }

    @Override
    public void onTabDocumentLoaded(String tag) {
        try {
            if (mPdfViewCtrlTabHostFragment != null && mPdfViewCtrlTabHostFragment.getCurrentPdfViewCtrlFragment() != null
                    && mPdfViewCtrlTabHostFragment.getCurrentPdfViewCtrlFragment().getPDFViewCtrl() != null) {
                pdfViewCtrl = mPdfViewCtrlTabHostFragment.getCurrentPdfViewCtrlFragment().getPDFViewCtrl();
                if (mDocToLoad != null) {
                    if (mPdfViewCtrlTabHostFragment.getCurrentPdfViewCtrlFragment() instanceof MyTabFragment) {
                        ((MyTabFragment)mPdfViewCtrlTabHostFragment.getCurrentPdfViewCtrlFragment()).setPdfDoc(mDocToLoad);
                    }
                    mDocToLoad = null;
                    invalidateOptionsMenu();
                } else {
                    mPdfDoc = pdfViewCtrl.getDoc();
                }
            }
            invalidateOptionsMenu();
            Log.i(TAG,"PDF onTabDocumentLoaded");
        }catch (Exception e){
            Log.i(TAG,"PDF onTabDocumentLoaded Exception");
        }
    }

    private class DownloadFileTask extends AsyncTask<Void, Void, PDFDoc> {
        AppCompatActivity context;
        ProgressDialog dialog;
        String url;
        public DownloadFileTask(AppCompatActivity context,String url){
            this.context = context;
            this.url = url;
            dialog = new ProgressDialog(context, ProgressDialog.THEME_DEVICE_DEFAULT_LIGHT);
        }

        @Override
        protected void onPreExecute() {
            super.onPreExecute();
            dialog.setMessage("Loading Please wait...");
            dialog.setCancelable(false);
            dialog.setCanceledOnTouchOutside(false);
            dialog.show();
        }

        @Override
        protected PDFDoc doInBackground(Void... voids) {
            PDFDoc pdfDoc = null;
            Response response =null;
            try {
                Log.i(TAG, "DownloadFileTask: "+url);
                Request OkHttpRequest = new Request.Builder()
                        .url(url)
                        .build();
                OkHttpClient.Builder httpClient = new OkHttpClient.Builder();
                OkHttpClient okHttpClient = httpClient.build();
                response = okHttpClient.newCall(OkHttpRequest).execute();
                if (response!=null && response.code() == 200) {
                    pdfbytes = response.body().bytes();
                    pdfDoc = new PDFDoc(pdfbytes);
                }
            }catch (Exception e){
                e.printStackTrace();
            }
            return pdfDoc;
        }

        @Override
        protected void onPostExecute(PDFDoc pdfDoc) {
            super.onPostExecute(pdfDoc);
            if (dialog != null && dialog.isShowing())
                dialog.dismiss();
            try {
                if (pdfDoc!=null) {
                    mDocToLoad = pdfDoc;
                    if (mPdfViewCtrlTabHostFragment.getCurrentPdfViewCtrlFragment() != null && mPdfViewCtrlTabHostFragment.getCurrentPdfViewCtrlFragment().isDocumentReady()) {
                        pdfViewCtrl = mPdfViewCtrlTabHostFragment.getCurrentPdfViewCtrlFragment().getPDFViewCtrl();
                        if (pdfViewCtrl != null) {
                            mDocToLoad = null;
                            if (mPdfViewCtrlTabHostFragment.getCurrentPdfViewCtrlFragment() instanceof MyTabFragment) {
                                ((MyTabFragment)mPdfViewCtrlTabHostFragment.getCurrentPdfViewCtrlFragment()).setPdfDoc(pdfDoc);
                            }
                            pdfViewCtrl.setPageViewMode(PDFViewCtrl.PageViewMode.FIT_WIDTH);
                            pdfViewCtrl.setPagePresentationMode(PDFViewCtrl.PagePresentationMode.SINGLE_CONT);
                            invalidateOptionsMenu();
                        }
                    }
                }
            }catch (Exception e){
                e.printStackTrace();
            }
        }
    }

    @Override
    public void onBackPressed() {
        mPdfViewCtrlTabHostFragment.removeTabAt(0);
        super.onBackPressed();

    }
}
